/*
Name: Mihir Leel
Course code: TEJ4M0
Date: January 16th, 2022
Description: This is my circuit for the Sad Cat project. What happens is that the cat comes close to the project, starts hearing bird chirp noises and grows interested
and then the cat must press the fuzzy decorated button and once pressed, lasers start going off and they rotate randomly using the servos and a random command all while
the bird chirping sound remains ON.

Arduino pins used: 3, 4, 5, 6, 7, 8, 9, 10, 11, 12 & 13

*/

//add public library
#include <Servo.h>  

// defines pins numbers for ultrasonic sensor
#define echoPin 6 
#define trigPin 7


//---------------------------------------------------------------------------------------------------------------------------------------------------------//
// This is the constant variables section - it has many constant integers set up and ready for use through the multiple functions
//---------------------------------------------------------------------------------------------------------------------------------------------------------//
Servo servo1; // initializes the servo to be called servo1
Servo servo2; // initializes the servo to be called servo1

const int buzz1 = 8; //this and next line sets the buzzer pins
const int buzz2 = 9;
const int button = 5; //sets the button's pin
int time = 0; //variable which will hold time which will tell how many time lasers will be allowed to be ON for

//define variables for buzzers
int count = 2;
int count2 = 2;
int i;

// defines variables for the ultrasonic sensor
long duration; //duration is a long time hence it is defined as a long variable
int distance; //same concept as variable duration



//**************************************************************
// setup function - this code runs once
//**************************************************************
void setup() 
{
  servo1.attach(3); //Servo connects to pin 3
  servo2.attach(4); //Servo connects to pin 4
  //note: servo pins can be interchanged with one another (ex. one servo could use either pin 3 or 4, and result will be the same)
  
  for (int x=10;x<14;x++)//loop uses to configure pins 10-13 which are used for the lasers
  {
  pinMode(x,OUTPUT);//declares laser pins as outputs
  }
  
  pinMode(buzz1,OUTPUT); //declare buzzer 1 as output
  pinMode(buzz2,OUTPUT); //declare buzzer 2 as output
  pinMode(echoPin, INPUT); // Sets the echoPin as an Input
  pinMode(trigPin, OUTPUT); // Sets the trigPin as an Output
  pinMode(button, INPUT_PULLUP); // Enable internal pull-up resistor on button, this saves time and decreases risk of button failure
}



//**************************************************************
// loop function - this code runs forever
//**************************************************************
void loop() 
{
  time=0; //ensures that the time variable is resetted whenever the code restarts
  ultrasonic(); //calls the ultrasonic method which will caclculate the distance from one entity to the ultrasonic sensor
  
  do{ //does this loop which the distance is greater than 50cm , done so code doesn't start when the cat's not near the project 
    
  ultrasonic(); //recalls ultrasonic method to constantly update the distance in case if cat decides to come close
  
  }while(distance>50); //distance changable, but be careful to keep it relatively close to 50cm as too much causes the music to start even as the cats not neaby as it monitors the other object's measurement
  // from previous comment: ex. changing to 100 caused music to go off constantly as the ultrasonic measured distance of wall instead of my hand (which I used to test)
  
  do{ //now since cat came close, now this loop plays while the cat hasn't pressed the obvious to press button
    chirp(); //plays the bird sound using the chirp function
  }while(digitalRead(button)==1); //condition set to digitalRead(button)==1 as the 1 represents the button NOT pressed since button is using pull up resistor
  
  for (int x=10;x<14;x++) //this loop uses x values of 10-13 to turn ON the lasers
  {
  digitalWrite(x,HIGH);//turns the lasers ON
  } 
  delay(1000); //optional line, recommended to allow testing for lasers before servo begins rotating
  
  while(time<3) //this while loops uses the time variable from the start to allow servos to rotate a total of 30 times before ending, time variable can be changed 
  {
    for(int y=0;y<11;y=y+1) //loop used to allow servos to rotate 10 quick times  
        {
          //we don't want the cat to quickly get bored by a standard pattern, hence we make each servo rotate randomly using the random command from 1-180 degrees 
          servo1.write(random(1,180)); //rotates servo 1 randomly
          servo2.write(random(1,180)); //rotates servo 2 randomly
      
          if ( (y % 3) == 0) //this basically says that if value of y is divisible by 3, then run this if statement
          {
            chirp(); //the chirp sound will be played at multiples of 3 using the y value
          }
          delay(2000); //waits 2 seconds to allow cat to try to catch the laser before rerotating the laser
        }
    time++; //increases time by 1 
  }
  
  for (int x=10;x<14;x++) //loop turns OFF pins 10-13 (Lasers)
  {
  digitalWrite(x,LOW);//turns OFF the lasers
  }     
  
  time=0; //resets time to 0 to allow cat to play again if desired by coming close and pressing the button
}



//---------------------------------------------------------------------------------------------------------------------------------------------------------//
// This is a function - It contains blocks of code which is used in my main, loop, function
//---------------------------------------------------------------------------------------------------------------------------------------------------------//
void ultrasonic() // Clears the trigPin to make sure the output stays clear and allows ultrasonic sensor to turn ON
{
  //next 5 lines of code (not comment included)work to make sure output is clear
  digitalWrite(trigPin, LOW);
  delayMicroseconds(2);
  // Sets the trigPin on HIGH state for 10 micro seconds
  digitalWrite(trigPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPin, LOW);
  
  // Reads the echoPin, returns the sound wave travel time in microseconds
  duration = pulseIn(echoPin, HIGH); 
  // Calculating the distance in cm formula (other ways to do it, but this way most efficent and gives most precise reading)
  distance= duration/58.2;
}



//---------------------------------------------------------------------------------------------------------------------------------------------------------//
// This is a function - It contains blocks of code which is used in my main, loop, function
//---------------------------------------------------------------------------------------------------------------------------------------------------------//
void chirp() //this is the code for the sound produced by the buzzers
{
  //MUST use all lines in this method to make the chirp sound (this was the most relative sound I could produce)
    while(count >0){     
      for(i=200;i>0;i--){  //i value is changable to experiment around with
        digitalWrite(buzz1,HIGH); //turns ON the first buzzer
        digitalWrite(buzz2,HIGH); //turns ON the second buzzer
        delayMicroseconds(i+200);
        digitalWrite(buzz1,LOW); //turns OFF the first buzzer
        digitalWrite(buzz2,LOW); //turns OFF the second buzzer
        delayMicroseconds(i+200);
      } 
      delay(100);
      count--;
      } 
  
  while(count2 >0){ 
    for(i=0;i<300;i++){ //i value is changable to experiment around with
      digitalWrite(buzz1,HIGH); //turns ON the first buzzer
      digitalWrite(buzz2,HIGH); //turns ON the second buzzer
      delayMicroseconds(i);
      digitalWrite(buzz1,LOW); //turns OFF the first buzzer
      digitalWrite(buzz2,LOW); //turns OFF the second buzzer
      delayMicroseconds(i);  
    } 
    count2--;
  }   
  
  //resets the values of count(s) back to 2, mandatory otherwise this method will constantly be skipped
  count = 2; 
  count2 = 2;
  
  delay(1000); //waits 1 second before moving further into the code
}